/*
 * Copyright (C) 2009 HBM Netherlands B.V.
 * Schutweg 15a
 * 5145 NP Waalwijk
 * The Netherlands
 * http://www.hbm.com
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>

#include "GEN_DAQ_API.h"

static void confirm_close(void);
static void terminate(const char *format, ...);

int main(void)
{
    char* ipaddress = "192.168.1.1";
    unsigned short portno = 8006;
    GHSConnectionHandle connection = 0;
    GHSReturnValue returnCode;
    uint32_t serverAPIVersion;
    double sampleRate = 0;
    GHSSlotID slotID = 'A';
    char *recorderType = NULL, *recorderName = NULL, *serialNumber = NULL, *firmwareVersion = NULL;

    printf("Connecting to %s:%u\n", ipaddress, portno);

    /* Connect to the mainframe "localhost" is also possible. */

    returnCode = GHSConnect(ipaddress, portno, &connection, &serverAPIVersion);
    if (returnCode == GHSReturnValue_APIMismatch)
    {
        terminate("Failed: client has API version %u, while mainframe has API version %u.\n",
        GHSGetClientAPIVersion(), serverAPIVersion);
    }
    else if (returnCode != GHSReturnValue_OK)
    {
        terminate("Failed: return code is %d.\n", returnCode);
    }

    printf("Connected\n");

    /* Get recorder information. */

    returnCode = GHSGetRecorderInformation(connection, slotID, &recorderType, &recorderName, &serialNumber, &firmwareVersion);
    if (returnCode != GHSReturnValue_OK)
    {
        terminate("Failed on GHSGetRecorderInformation: return code is %d.\n", returnCode);
    }

    printf("\nRecorder slot: %c\n", slotID);

    /* Get sample rate. */

    returnCode = GHSGetSampleRate(connection, slotID, &sampleRate);
    if (returnCode != GHSReturnValue_OK)
    {
        terminate("Failed on GHSGetSampleRate: return code is %d.\n", returnCode);
    }

    /* Set sample rate. */

    returnCode = GHSSetSampleRate(connection, slotID, sampleRate);
    if (returnCode != GHSReturnValue_OK)
    {
        terminate("Failed on GHSSetSampleRate: return code is %d.\n", returnCode);
    }

    printf("\nSample-rate: %.2f\n", sampleRate);

    /* Show recorder information. */

    printf("\nRecorder Information:\n");
    printf("Type: %s\n", recorderType);
    printf("Name: %s\n", recorderName);
    printf("Serial number: %s\n", serialNumber);
    printf("Firmware version: %s\n\n", firmwareVersion);

    /* Disconnect from the mainframe (this enables Perception or another GHS API client to connect). */

    returnCode = GHSDisconnect(connection);
    if (returnCode != GHSReturnValue_OK)
    {
        terminate("Failed on GHSDisconnect: return code is %d.\n", returnCode);
    }

    printf("Disconnected\n");

    confirm_close();
    return EXIT_SUCCESS;
}

void confirm_close(void)
{
    printf("Press ENTER to close this example program.\n");
    (void)getchar();
}

void terminate(const char *format, ...)
{
    va_list args;

    va_start(args, format);
    vprintf(format, args);
    va_end(args);

    confirm_close();

    exit(EXIT_FAILURE);
}
