/*
* Copyright (C) 2009 HBM Netherlands B.V.
* Schutweg 15a
* 5145 NP Waalwijk
* The Netherlands
* http://www.hbm.com
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*/

#if defined(__unix__) && !defined(_POSIX_C_SOURCE)
#define _POSIX_C_SOURCE 200809L
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "jsonrpc.h"

#if defined(_MSC_VER)
#define strdup _strdup
#endif

#if defined(__MINGW32__)
char *strdup(const char *s);
#endif

const char *JsonRpcCreateRequest(const char *method, int requestID, JsonRpcNamedParameter *params, size_t numParams)
{
    JSON_Value *root_value = json_value_init_object();
    JSON_Object *root_object = json_value_get_object(root_value);
    char *serialized_string = NULL;
    size_t i;

    json_object_set_string(root_object, "jsonrpc", "2.0");
    json_object_set_string(root_object, "method", method);

    for (i = 0; i < numParams; i++)
    {
        JsonRpcNamedParameter currentParam = params[i];

        char paramsElement[128];
        strcpy(paramsElement, "params.");
        strcat(paramsElement, currentParam.key);

        switch (currentParam.vType)
        {
        case VT_int:
            json_object_dotset_number(root_object, paramsElement, (double)currentParam.value.iValue);
            break;
        case VT_string:
            json_object_dotset_string(root_object, paramsElement, currentParam.value.sValue);
            break;
        case VT_double:
            json_object_dotset_number(root_object, paramsElement, currentParam.value.dValue);
            break;
        case VT_array:
            /* To do */
            break;
        }
    }

    json_object_set_number(root_object, "id", requestID);
    serialized_string = json_serialize_to_string(root_value);
    json_value_free(root_value);
    return serialized_string;
}

GHSReturnValue JsonRpcCheckForErrors(JSON_Object *root_object, int expectedRequestID)
{
    int errorCode;

    /* Checks id is null on JSON */
    int isNull = json_object_has_value_of_type(root_object, "id", JSONNull);

    /* If it is null then a critical error happened */
    if (isNull)
    {
        /* If retreivedRequestID is null means some critical error was received by the client */
        /* These can be: Invalid Request, Parse Error, ...*/

        /* Check if the JSON has error key in it */
        if (json_object_has_value_of_type(root_object, "error", JSONObject))
        {
            /* Retrieve error object from the JSON */
            JSON_Object *error_object = json_object_get_object(root_object, "error");

            /*Retrieves the error code from the JSON */
            errorCode = (int) json_object_get_number(error_object, "code");

            /* Depending on the code the message sent to application can differ */
            switch(errorCode)
            {
                /* Invalid JSON Request */
                case -32700:
                    return GHSReturnValue_InvalidJSONFormat;
                    break;

                /* Undefined Error Message */
                default:
                    return GHSReturnValue_UnkownErrorMessage;
            }
        }
        else
        {
            return GHSReturnValue_NOK;
        }
    }
    else
    {
        /* In the current design (send one request, wait for one response)
         * This cannot happen
         * Check anyways for future
         */

        double retreivedRequestID = json_object_dotget_number(root_object, "id");
        if (retreivedRequestID != expectedRequestID)
        {
            return GHSReturnValue_NOK; /* TODO: Better ReturnValue for transport errors */
        }

    }
    /* Determine how to handle the message depending on what's in it */
    /* Non Critical JSON errors such as: Invalid Method, Invalid Parameters for Method, ... */
    /* Example would be: */
    /* {"jsonrpc": "2.0", "error": {"code": -32601, "message": "Method not found"}, "id": 2} */
    if (json_object_has_value_of_type(root_object, "error", JSONObject))
    {
        /* Retrieve error object from the JSON */
        JSON_Object *error_object = json_object_get_object(root_object, "error");

        /*Retrieves the error code from the JSON */
        errorCode = (int) json_object_get_number(error_object, "code");

        /* Depending on the code the message sent to application can differ */
        switch(errorCode)
        {
            /* Invalid Method */
            case -32601:
                return GHSReturnValue_MethodNotFound;
                break;
            /* Undefined Error Message */
            default:
                return GHSReturnValue_UnkownErrorMessage;
        }
    }
    return GHSReturnValue_OK;
}

GHSReturnValue JsonRpcParseResponse(const char *message, int expectedRequestID, JsonRpcNamedParameter **params, size_t *numParams)
{
    /* We expect one of the following 3 type of responses
     * {"jsonrpc": "2.0", "result": 7, "id": 1}
     * {"jsonrpc": "2.0", "error": {"code": -32601, "message": "Method not found"}, "id": 2}
     * {"jsonrpc": "2.0", "result": {"GHSReturnValue": 3, "message": "These are not the slots you are looking for"}, "id": 3};
     */

    JSON_Value *root_value;
    JSON_Object *root_object;
    GHSReturnValue returnValue;

    if (numParams)
        *numParams = 0;
    if (params)
        *params = 0;

    /* Check if we received a JSON Object */
    root_value = json_parse_string(message);
    if (json_value_get_type(root_value) != JSONObject)
    {
        json_value_free(root_value);
        return GHSReturnValue_NOK;
    }

    /* Check if the JSON Object contains the expected ID */
    root_object = json_object(root_value);

    /* Check if retrevied JSON has any error code in it */
    returnValue = JsonRpcCheckForErrors(root_object, expectedRequestID);

    /* Error was found when checking the JSON for errors */
    if (returnValue != GHSReturnValue_OK)
    {
        json_object_clear(root_object);
        json_value_free(root_value);
        return returnValue;
    }

    if (json_object_has_value_of_type(root_object, "result", JSONNumber))
    {
        /* {"jsonrpc": "2.0", "result": 7, "id": 1}" */

        /* A result number in the response MUST be a GHSReturnValue integer, return that */
        returnValue = (GHSReturnValue)json_object_get_number(root_object, "result");

    }
    else if (json_object_has_value_of_type(root_object, "result", JSONObject))
    {
        /* {"jsonrpc": "2.0", "result": {"GHSReturnValue": 3, "message": "These are not the slots you are looking for"}, "id": 3} */

        JSON_Object *result_object = json_object_get_object(root_object, "result");
        size_t numberOfParameters = json_object_get_count(result_object);

        if (params && numParams)
        {
            size_t i;

            *numParams = numberOfParameters;
            *params = malloc(sizeof(JsonRpcNamedParameter)*numberOfParameters);

            for (i = 0; i < numberOfParameters; i++)
            {
                const char *name = json_object_get_name(result_object, i);
                JSON_Value *value = json_object_get_value_at(result_object, i);
                JsonRpcNamedParameter thisParam = JsonRpcCreateNamedParameter(name, value);
                JsonRpcNamedParameter *param = &((*params)[i]);

                memcpy(param, &thisParam, sizeof(JsonRpcNamedParameter));

                if (thisParam.vType == VT_string)
                {
                    param->value.sValue = strdup(json_value_get_string(value));
                }

                param->key = strdup(thisParam.key);
            }
        }

        /* A result object in the response MUST have a GHSReturnValue named parameter, return that */
        if (result_object != NULL)
        {
            returnValue = (GHSReturnValue)json_object_get_number(result_object, "GHSReturnValue");
            json_object_clear(result_object);
        }
        else
        {
            returnValue = GHSReturnValue_NOK;
        }
    }
    else
    {
        /* Invalid JSON-RPC response */
        returnValue = GHSReturnValue_NOK;
    }

    json_object_clear(root_object);
    json_value_free(root_value);

    return returnValue;
}

JsonRpcNamedParameter JsonRpcCreateNamedParameter(const char *key, const JSON_Value *value)
{
    switch (json_value_get_type(value))
    {
    case JSONNumber:
        return JsonRpcCreateDoubleNamedParameter(key, json_value_get_number(value));
    case JSONString:
        return JsonRpcCreateStringNamedParameter(key, json_value_get_string(value));
    case JSONBoolean:
        return JsonRpcCreateDoubleNamedParameter(key, json_value_get_boolean(value));
    case JSONArray:
        return JsonRpcCreateArrayParameter(key, json_value_get_array(value));
    default:
        /* Not supported yet */
        return JsonRpcCreateStringNamedParameter("", "");
    }
}

JsonRpcNamedParameter JsonRpcCreateDoubleNamedParameter(const char *key, double value)
{
    JsonRpcNamedParameter param;
    param.key = key;
    param.vType = VT_double;
    param.value.dValue = value;
    return param;
}

JsonRpcNamedParameter JsonRpcCreateIntNamedParameter(const char *key, int value)
{
    JsonRpcNamedParameter param;
    param.key = key;
    param.vType = VT_int;
    param.value.iValue = value;

    return param;
}

JsonRpcNamedParameter JsonRpcCreateStringNamedParameter(const char *key, const char *value)
{
    JsonRpcNamedParameter param;
    param.key = key;
    param.vType = VT_string;
    param.value.sValue = value;
    return param;
}

JsonRpcNamedParameter JsonRpcCreateArrayParameter(const char *key, const JSON_Array *value)
{
    size_t i;
    size_t count;
    float *arr_to_point;

    JsonRpcNamedParameter param;
    param.key = key;
    param.vType = VT_array;

    count = json_array_get_count(value);

    arr_to_point = malloc(count * sizeof(float));
    param.value.arValue = arr_to_point;

    for (i = 0; i < count; i++)
    {
        param.value.arValue[i] = (float)json_array_get_number(value, i);
    }
    return param;
}
